Package: bbk
Title: Client for the Bundesbank API
Version: 0.2.0
Authors@R: 
    person("Maximilian", "Muecke", , "muecke.maximilian@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0009-0000-9432-9795"))
Description: Download data and metadata from the 'Bundesbank SDMX Web
    Service API' found at
    <https://www.bundesbank.de/en/statistics/time-series-databases/help-for-sdmx-web-service/web-service-interface-data>.
License: MIT + file LICENSE
URL: https://m-muecke.github.io/bbk/, https://github.com/m-muecke/bbk
BugReports: https://github.com/m-muecke/bbk/issues
Depends: R (>= 4.1.0)
Imports: httr2, stats, utils, xml2
Suggests: ggplot2, scales, testthat (>= 3.0.0), tibble
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.3.1
NeedsCompilation: no
Packaged: 2024-04-28 09:29:25 UTC; mmuecke
Author: Maximilian Muecke [aut, cre] (<https://orcid.org/0009-0000-9432-9795>)
Maintainer: Maximilian Muecke <muecke.maximilian@gmail.com>
Repository: CRAN
Date/Publication: 2024-04-30 09:20:09 UTC
Built: R 4.2.3; ; 2024-04-30 11:43:26 UTC; unix
