% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/srb.R
\name{srb_calendar}
\alias{srb_calendar}
\title{Fetch Sveriges Riksbank (SRb) calendar days}
\source{
\url{https://developer.api.riksbank.se/}
}
\usage{
srb_calendar(start_date, end_date = NULL)
}
\arguments{
\item{start_date}{(\code{Date(1)} | \code{character(1)})\cr
Start date of the range (e.g., \code{"2024-01-01"}).}

\item{end_date}{(\code{NULL} | \code{Date(1)} | \code{character(1)})\cr
End date of the range. If \code{NULL}, data up to the latest available date is returned.
Default \code{NULL}.}
}
\value{
A \code{\link[data.table:data.table]{data.table::data.table()}} with the calendar day information.
}
\description{
Retrieve Swedish banking calendar information from the Sveriges Riksbank SWEA API.
}
\examples{
\dontshow{if (curl::has_internet()) withAutoprint(\{ # examplesIf}
\donttest{
srb_calendar("2024-01-01", "2024-01-31")
}
\dontshow{\}) # examplesIf}
}
\seealso{
Other metadata:
\code{\link[=bbk_dimension]{bbk_dimension()}},
\code{\link[=bbk_metadata]{bbk_metadata()}},
\code{\link[=bdp_dataset]{bdp_dataset()}},
\code{\link[=bdp_dimension]{bdp_dimension()}},
\code{\link[=bdp_domain]{bdp_domain()}},
\code{\link[=bdp_series]{bdp_series()}},
\code{\link[=bis_dimension]{bis_dimension()}},
\code{\link[=bis_metadata]{bis_metadata()}},
\code{\link[=boj_metadata]{boj_metadata()}},
\code{\link[=ecb_dimension]{ecb_dimension()}},
\code{\link[=ecb_metadata]{ecb_metadata()}},
\code{\link[=nob_dimension]{nob_dimension()}},
\code{\link[=nob_metadata]{nob_metadata()}},
\code{\link[=onb_dimension]{onb_dimension()}},
\code{\link[=onb_frequency]{onb_frequency()}},
\code{\link[=onb_hierarchy]{onb_hierarchy()}},
\code{\link[=onb_metadata]{onb_metadata()}},
\code{\link[=onb_toc]{onb_toc()}},
\code{\link[=snb_dimension]{snb_dimension()}},
\code{\link[=srb_series]{srb_series()}}
}
\concept{metadata}
