% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boc.R
\name{boc_metadata}
\alias{boc_metadata}
\title{Fetch Bank of Canada (BoC) metadata (details)}
\source{
\url{https://www.bankofcanada.ca/valet/docs}
}
\usage{
boc_metadata(group_name = NULL, series_name = NULL)
}
\arguments{
\item{group_name}{(\code{NULL} | \code{character(1)})\cr
Name of the group. Only one of \code{group_name} or \code{series_name} can be used.}

\item{series_name}{(\code{NULL} | \code{character()})\cr
Name of the series.}
}
\value{
A \code{\link[data.table:data.table]{data.table::data.table()}} with the requested data.
}
\description{
Fetch Bank of Canada (BoC) metadata (details)
}
\examples{
\dontrun{
boc_metadata(group_name = "FX_RATES_DAILY")
boc_metadata(series_name = "FXUSDCAD")
}
}
