% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cache.R
\name{cache}
\alias{cache}
\alias{bbk_cache_dir}
\alias{bbk_cache_clear}
\title{Get or manage the bbk API cache}
\usage{
bbk_cache_dir()

bbk_cache_clear()
}
\description{
\code{bbk_cache_dir()} returns the path where cached API responses are stored.
\code{bbk_cache_clear()} clears all cached responses.
}
\details{
The cache is only used when enabled with \code{options(bbk.cache = TRUE)}.
Cached responses are stored for 1 day by default, but this can be customized with
\code{options(bbk.cache_max_age = seconds)}.
}
\examples{
\dontrun{
# enable caching
options(bbk.cache = TRUE)

# view cache location
bbk_cache_dir()

# clear the cache
bbk_cache_clear()
}
}
