% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bbk-package.R
\docType{package}
\name{bbk-package}
\alias{bbk}
\alias{bbk-package}
\title{bbk: Client for Central Bank APIs}
\description{
A client for retrieving data and metadata from central bank APIs including 'Banco de España' (BdE), 'Banco de Portugal' (BdP), 'Bank for International Settlements' (BIS), 'Bank of Canada' (BoC), 'Bank of England' (BoE), 'Bank of Japan' (BoJ), 'Banque de France' (BdF), 'Deutsche Bundesbank' (BBk), 'European Central Bank' (ECB), 'National Bank of Poland' (NBP), 'Norges Bank' (NoB), 'Oesterreichische Nationalbank' (OeNB), 'Sveriges Riksbank' (SRb), and 'Swiss National Bank' (SNB).
}
\seealso{
Useful links:
\itemize{
  \item \url{https://m-muecke.github.io/bbk/}
  \item \url{https://github.com/m-muecke/bbk}
  \item Report bugs at \url{https://github.com/m-muecke/bbk/issues}
}

}
\author{
\strong{Maintainer}: Maximilian Mücke \email{muecke.maximilian@gmail.com} (\href{https://orcid.org/0009-0000-9432-9795}{ORCID})

Authors:
\itemize{
  \item Maximilian Mücke \email{muecke.maximilian@gmail.com} (\href{https://orcid.org/0009-0000-9432-9795}{ORCID})
}

}
\keyword{internal}
