% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bbk.R
\name{bbk_dimension}
\alias{bbk_dimension}
\title{Fetch Deutsche Bundesbank (BBk) dimensions}
\source{
\url{https://www.bundesbank.de/en/statistics/time-series-databases/help-for-sdmx-web-service/web-service-interface-metadata}
}
\usage{
bbk_dimension(id)
}
\arguments{
\item{id}{(\code{character(1)})\cr
The id of the data structure definition to query (e.g., \code{"BBK_BBSIS"}).}
}
\value{
A \code{\link[data.table:data.table]{data.table::data.table()}} with columns:
\item{id}{The dimension id (e.g., \code{"BBK_STD_FREQ"}, \code{"BBK_STD_AREA"})}
\item{position}{The position of the dimension in the series key}
\item{codelist}{The id of the associated codelist}
}
\description{
Retrieve the dimension structure for a given dataflow from the Bundesbank SDMX Web Service.
}
\examples{
\dontshow{if (curl::has_internet()) withAutoprint(\{ # examplesIf}
\donttest{
bbk_dimension("BBK_ERX")
}
\dontshow{\}) # examplesIf}
}
\seealso{
Other metadata:
\code{\link[=bbk_metadata]{bbk_metadata()}},
\code{\link[=bdp_dataset]{bdp_dataset()}},
\code{\link[=bdp_dimension]{bdp_dimension()}},
\code{\link[=bdp_domain]{bdp_domain()}},
\code{\link[=bdp_series]{bdp_series()}},
\code{\link[=bis_dimension]{bis_dimension()}},
\code{\link[=bis_metadata]{bis_metadata()}},
\code{\link[=boj_metadata]{boj_metadata()}},
\code{\link[=ecb_dimension]{ecb_dimension()}},
\code{\link[=ecb_metadata]{ecb_metadata()}},
\code{\link[=nob_dimension]{nob_dimension()}},
\code{\link[=nob_metadata]{nob_metadata()}},
\code{\link[=onb_dimension]{onb_dimension()}},
\code{\link[=onb_frequency]{onb_frequency()}},
\code{\link[=onb_hierarchy]{onb_hierarchy()}},
\code{\link[=onb_metadata]{onb_metadata()}},
\code{\link[=onb_toc]{onb_toc()}},
\code{\link[=snb_dimension]{snb_dimension()}},
\code{\link[=srb_calendar]{srb_calendar()}},
\code{\link[=srb_series]{srb_series()}}
}
\concept{metadata}
