% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/onb.R
\name{onb_hierarchy}
\alias{onb_hierarchy}
\title{Fetch Österreichische Nationalbank (OeNB) hierarchy}
\usage{
onb_hierarchy(hier_id, lang = "en")
}
\arguments{
\item{hier_id}{(\code{integer(1)})\cr
Hierarchy id to query.}

\item{lang}{(\code{character(1)})\cr
Language to query. Default \code{"en"}.}
}
\value{
A \code{\link[data.table:data.table]{data.table::data.table()}} with the requested data.
}
\description{
Fetch Österreichische Nationalbank (OeNB) hierarchy
}
\examples{
\dontshow{if (curl::has_internet()) withAutoprint(\{ # examplesIf}
\donttest{
onb_hierarchy(hier_id = 11)
}
\dontshow{\}) # examplesIf}
}
\seealso{
Other metadata:
\code{\link[=bbk_dimension]{bbk_dimension()}},
\code{\link[=bbk_metadata]{bbk_metadata()}},
\code{\link[=bdp_dataset]{bdp_dataset()}},
\code{\link[=bdp_dimension]{bdp_dimension()}},
\code{\link[=bdp_domain]{bdp_domain()}},
\code{\link[=bdp_series]{bdp_series()}},
\code{\link[=bis_dimension]{bis_dimension()}},
\code{\link[=bis_metadata]{bis_metadata()}},
\code{\link[=boj_metadata]{boj_metadata()}},
\code{\link[=ecb_dimension]{ecb_dimension()}},
\code{\link[=ecb_metadata]{ecb_metadata()}},
\code{\link[=nob_dimension]{nob_dimension()}},
\code{\link[=nob_metadata]{nob_metadata()}},
\code{\link[=onb_dimension]{onb_dimension()}},
\code{\link[=onb_frequency]{onb_frequency()}},
\code{\link[=onb_metadata]{onb_metadata()}},
\code{\link[=onb_toc]{onb_toc()}},
\code{\link[=snb_dimension]{snb_dimension()}},
\code{\link[=srb_calendar]{srb_calendar()}},
\code{\link[=srb_series]{srb_series()}}
}
\concept{metadata}
