% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/srb.R
\name{srb_cross_rates}
\alias{srb_cross_rates}
\title{Fetch Sveriges Riksbank (SRb) cross rates}
\source{
\url{https://developer.api.riksbank.se/}
}
\usage{
srb_cross_rates(series1, series2, start_date, end_date = NULL)
}
\arguments{
\item{series1}{(\code{character(1)})\cr
The first series ID (e.g., \code{"SEKUSDPMI"}).}

\item{series2}{(\code{character(1)})\cr
The second series ID (e.g., \code{"SEKEURPMI"}).}

\item{start_date}{(\code{NULL} | \code{Date(1)} | \code{character(1)})\cr
Start date of the data (e.g., \code{"2024-01-01"}). If \code{NULL}, all available data is returned.
Default \code{NULL}.}

\item{end_date}{(\code{NULL} | \code{Date(1)} | \code{character(1)})\cr
End date of the data, in the same format as start_date. If \code{NULL}, data up to the latest
available date is returned. Default \code{NULL}.}
}
\value{
A \code{\link[data.table:data.table]{data.table::data.table()}} with the cross rate data.
}
\description{
Compute cross exchange rates between two currency series from the Sveriges Riksbank SWEA API.
}
\examples{
\dontshow{if (curl::has_internet()) withAutoprint(\{ # examplesIf}
\donttest{
# USD/EUR cross rate
srb_cross_rates("SEKUSDPMI", "SEKEURPMI", start_date = "2024-01-01", end_date = "2024-01-31")
}
\dontshow{\}) # examplesIf}
}
\seealso{
Other data:
\code{\link[=bbk_data]{bbk_data()}},
\code{\link[=bbk_series]{bbk_series()}},
\code{\link[=bde_data]{bde_data()}},
\code{\link[=bde_latest]{bde_latest()}},
\code{\link[=bdf_codelist]{bdf_codelist()}},
\code{\link[=bdf_data]{bdf_data()}},
\code{\link[=bdf_dataset]{bdf_dataset()}},
\code{\link[=bdp_data]{bdp_data()}},
\code{\link[=bis_data]{bis_data()}},
\code{\link[=boc_data]{boc_data()}},
\code{\link[=boe_data]{boe_data()}},
\code{\link[=boj_data]{boj_data()}},
\code{\link[=ecb_data]{ecb_data()}},
\code{\link[=nbp_fx_rates]{nbp_fx_rates()}},
\code{\link[=nbp_gold]{nbp_gold()}},
\code{\link[=nob_data]{nob_data()}},
\code{\link[=onb_data]{onb_data()}},
\code{\link[=snb_data]{snb_data()}},
\code{\link[=srb_data]{srb_data()}}
}
\concept{data}
