% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bdf.R
\name{bdf_codelist}
\alias{bdf_codelist}
\title{Fetch Banque de France (BdF) codelists}
\source{
\url{https://webstat.banque-france.fr/en/pages/guide-migration-api/}
}
\usage{
bdf_codelist(..., lang = "en")
}
\arguments{
\item{...}{(\code{any})\cr
Extra arguments appended to the API request.
Combined with the default arguments with \code{\link[=modifyList]{modifyList()}}.}

\item{lang}{(\code{character(1)})\cr
Language to query. Default \code{"en"}.}
}
\value{
A \code{\link[data.table:data.table]{data.table::data.table()}} with the requested data.
}
\description{
Fetch Banque de France (BdF) codelists
}
\examples{
\dontrun{
bdf_codelist()

# filter for a specific codelist
bdf_codelist(where = "codelist_id = 'CL_FREQ'")
}
}
\seealso{
Other data:
\code{\link[=bbk_data]{bbk_data()}},
\code{\link[=bbk_series]{bbk_series()}},
\code{\link[=bde_data]{bde_data()}},
\code{\link[=bde_latest]{bde_latest()}},
\code{\link[=bdf_data]{bdf_data()}},
\code{\link[=bdf_dataset]{bdf_dataset()}},
\code{\link[=bdp_data]{bdp_data()}},
\code{\link[=bis_data]{bis_data()}},
\code{\link[=boc_data]{boc_data()}},
\code{\link[=boe_data]{boe_data()}},
\code{\link[=boj_data]{boj_data()}},
\code{\link[=ecb_data]{ecb_data()}},
\code{\link[=nbp_fx_rates]{nbp_fx_rates()}},
\code{\link[=nbp_gold]{nbp_gold()}},
\code{\link[=nob_data]{nob_data()}},
\code{\link[=onb_data]{onb_data()}},
\code{\link[=snb_data]{snb_data()}},
\code{\link[=srb_cross_rates]{srb_cross_rates()}},
\code{\link[=srb_data]{srb_data()}}
}
\concept{data}
